@@ -40,13 +40,14 @@ function (tcl.list)
     if (is.numeric(rownumbers) &&
         length(rownumbers) != nrow(dataframe)) 
         stop("rownumbers argument must be TRUE, FALSE or have length nrow(dataframe)")
-    require(tcltk) || stop("Tcl/Tk support is absent")
+#    require(tcltk) || stop("Tcl/Tk support is absent")
     oldwidth <- options()$width
     options(width = 10000)
-    conn <- textConnection("zz", "w")
+    conn <- file()
     sink(conn)
     print(dataframe)
     sink()
+    zz <- scan(conn, sep = "\n", what = character(0), quiet = TRUE)
     close(conn)
     options(width = oldwidth)
     base <- tktoplevel()
@@ -64,7 +65,6 @@ function (tcl.list)
     else rowname.text <- row.names(dataframe)
     namewidth = max(nchar(rowname.text))
     yy <- substring(zz, 2 + max(nchar(row.names(dataframe))))
-    rm(zz, envir = .GlobalEnv)
     datawidth <- max(nchar(yy))
     winwidth <- min(1 + datawidth, maxwidth)
     hdr <- tktext(base,
@@ -199,7 +199,7 @@ function (tcl.list)
     }
     tkgrid.rowconfigure(base, 1, weight = 1)
     tkgrid.columnconfigure(base, 1, weight = 1)
-    tkwm.maxsize(base, 1+datawidth, nrows)
-    tkwm.minsize(base, 1+nchar(names(dataframe)[1]), 1)
+    tkwm.maxsize(base, 1 + datawidth, nrows)
+    tkwm.minsize(base, 1 + nchar(names(dataframe)[1]), 1)
     invisible(NULL)
 }
